use telers::{
    event::{telegram::HandlerResult, EventReturn},
    types::{Message, User},
    Bot,
};

use tokio::time::{sleep, Duration as DurationSleep};

use crate::{
    types::structs::{
        handler_entity::HandlerEntity,
        message_sender::{MessageSender, MessageSenderBuilder},
    },
    utils::telegram::try_do::try_restrict,
};

use crate::utils::telegram::ban_member::ban_chat_member;

pub async fn ban(handler_entity: HandlerEntity, chat_id: i64, delay: u64) -> HandlerResult {
    let (bot, message, sender): (Bot, Message, MessageSenderBuilder) = (
        handler_entity.bot_instance,
        handler_entity.message_reciever,
        handler_entity.message_sender_builder,
    );

    let reply_to: Message;

    if let Some(msg) = message.reply_to_message() {
        reply_to = msg.clone();
    } else {
        sender
            .reply_to(message.id())
            .text("Ответьте на сообщение, чтобы забанить.")
            .build()
            .send(&bot)
            .await?;
        return Ok(EventReturn::Cancel);
    }

    let user: &User = reply_to.from().unwrap();

    sleep(DurationSleep::from_millis(delay)).await;

    let future = || async { ban_chat_member(&bot, user.id, chat_id).await };
    let demote_args: (&Bot, i64, i64) = (&bot, user.id, chat_id);
    let failure_sender: MessageSender = sender
        .clone()
        .text("Невозможно забанить участника чата, демотните и попробуйте снова")
        .build();

    if try_restrict(future, demote_args, failure_sender)
        .await
        .is_err()
    {
        Ok(EventReturn::Cancel)
    } else {
        let banned_user_name: String = reply_to.from().unwrap().clone().username.unwrap();

        sender
            .reply_to(message.id())
            .text(format!("Пользователь {} забанен.", banned_user_name))
            .build()
            .send(&bot)
            .await?;

        Ok(EventReturn::Finish)
    }
}
